# -*- coding: utf-8 -*-

# Copyright (C) 2023 Ross Scroggs All Rights Reserved.
#
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""GAM extended library versions

"""

GAM_VER_LIBS = [
  'chardet',
  'cryptography',
  'filelock',
  'google-api-python-client',
  'google-auth-httplib2',
  'google-auth-oauthlib',
  'google-auth',
  'lxml',
  'httplib2',
  'passlib',
  'pathvalidate',
  'pyscard',
  'python-dateutil',
  'yubikey-manager',
]
